'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.requireLogin = exports.CONFIG_FILE_PATH = undefined;

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

let requireLogin = exports.requireLogin = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* () {
    const token = getBitbucketToken();

    if (!token || token.length < 1) {
      console.log(_chalk2.default.yellow('This feature requires bitbucket access.'));
      console.log(_chalk2.default.cyan('Let\'s login to your account!'));
      return runSetup();
    }

    return _promise2.default.resolve('success');
  });

  return function requireLogin() {
    return _ref.apply(this, arguments);
  };
})();

exports.getConfig = getConfig;
exports.getBitbucketToken = getBitbucketToken;
exports.getBitbucketUsername = getBitbucketUsername;
exports.requireSmartCommits = requireSmartCommits;
exports.getJiraHost = getJiraHost;
exports.getJiraUsername = getJiraUsername;
exports.getJiraToken = getJiraToken;
exports.promptUserSetup = promptUserSetup;
exports.runSetup = runSetup;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _args = require('./args');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CONFIG_FILE_PATH = exports.CONFIG_FILE_PATH = `${_args.HOME}/.hiflow`;

function writeConfigFile(path, content) {
  return new _promise2.default((resolve, reject) => _fs2.default.writeFile(path, content, 'utf8', err => err ? reject(err) : resolve()));
}

function formatConfigForSave(configJSON = {}) {
  return (0, _keys2.default)(configJSON).reduce((memo, curr) => {
    memo.push(`${curr.trim()}=${configJSON[curr].trim()}`);
    return memo;
  }, []).join('\n');
}

function getConfigFile() {
  if (_fs2.default.existsSync(CONFIG_FILE_PATH)) {
    return _fs2.default.readFileSync(CONFIG_FILE_PATH, 'utf8');
  }

  throw new Error('Config not found');
}

function parseConfig(settings = '') {
  return settings.split('\n').reduce((memo, curr) => {
    const key = curr.substring(0, curr.indexOf('=')).trim();
    const value = curr.substring(curr.indexOf('=') + 1).trim();

    if (key && value) {
      return (0, _extends3.default)({}, memo, {
        [key]: value
      });
    }
    return memo;
  }, {});
}

function getConfig() {
  try {
    return parseConfig(getConfigFile()) || {};
  } catch (e) {
    return {};
  }
}

function getBitbucketToken() {
  return getConfig().BITBUCKET_TOKEN;
}

function getBitbucketUsername() {
  return getConfig().BITBUCKET_USERNAME;
}

function requireSmartCommits() {
  return getConfig().SMART_COMMITS === 'always';
}

function getJiraHost() {
  return getConfig().JIRA_HOST;
}

function getJiraUsername() {
  return getConfig().JIRA_USERNAME;
}

function getJiraToken() {
  return getConfig().JIRA_TOKEN;
}

function promptUserSetup() {
  return _inquirer2.default.prompt([{
    type: 'input',
    name: 'username',
    message: 'What\'s your bitbucket username?',
    default: getBitbucketUsername(),
    validate: val => !!val,
    filter: val => val.trim(),
    when: () => true
  }, {
    type: 'password',
    name: 'password',
    message: 'What\'s your bitbucket password?',
    validate: val => !!val,
    filter: val => val.trim(),
    when: () => true
  }, {
    type: 'confirm',
    name: 'smartcommits',
    message: 'Require all commits to use bitbucket "smart" commits?',
    validate: val => !!val,
    filter: val => val.trim(),
    when: () => true
  }]
  //    {
  //      type: 'input',
  //      name: 'jirahost',
  //      message: 'What\'s your JIRA host name?',
  //      default: getJiraHost(),
  //      validate: val => !!val,
  //      filter: val => val.trim(),
  //      when: () => true,
  //    },
  //    {
  //      type: 'input',
  //      name: 'jirausername',
  //      default: getJiraUsername(),
  //      message: 'What\'s your JIRA username?',
  //      validate: val => !!val,
  //      filter: val => val.trim(),
  //      when: () => true,
  //    },
  //    {
  //      type: 'password',
  //      name: 'jirapassword',
  //      message: 'What\'s your JIRA password?',
  //      validate: val => !!val,
  //      filter: val => val.trim(),
  //      when: () => true,
  //    },
  );
}

function createToken(username, password) {
  return Buffer.from(`${username}:${password}`).toString('base64');
}

function handlePrompt({
  username,
  password,
  smartcommits
  //  jirahost,
  //  jirausername,
  //  jirapassword,
}) {
  return writeConfigFile(CONFIG_FILE_PATH, formatConfigForSave((0, _extends3.default)({}, getConfig(), {
    BITBUCKET_USERNAME: username,
    BITBUCKET_TOKEN: createToken(username, password),
    SMART_COMMITS: smartcommits ? 'always' : 'optional'
    //    JIRA_HOST: jirahost,
    //    JIRA_USERNAME: jirausername,
    //    JIRA_TOKEN: createToken(jirausername, jirapassword),
  })));
}

function runSetup() {
  return promptUserSetup().then(handlePrompt).then(() => {
    console.log(_chalk2.default.cyan('Let\'s do this!'));
  }).catch(() => {});
}
//# sourceMappingURL=config.js.map