'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getVersion = getVersion;
exports.getBranch = getBranch;
exports.getRemoteURL = getRemoteURL;
exports.getRemoteUsernameFromURL = getRemoteUsernameFromURL;
exports.getRemoteUsername = getRemoteUsername;
exports.getRemoteRepositoryNameFromURL = getRemoteRepositoryNameFromURL;
exports.getRemoteRepositoryName = getRemoteRepositoryName;
exports.pullAndRebase = pullAndRebase;
exports.checkoutBranch = checkoutBranch;
exports.createBranch = createBranch;
exports.createCommit = createCommit;

var _child_process = require('child_process');

function getVersion() {
  return (0, _child_process.execSync)('git describe --tags --abbrev=0', { encoding: 'utf8' }).trim();
}

function getBranch() {
  return (0, _child_process.execSync)('git rev-parse --abbrev-ref HEAD', { encoding: 'utf8' }).trim();
}

function getRemoteURL() {
  return (0, _child_process.execSync)('git config --get remote.origin.url', { encoding: 'utf8' }).trim();
}

function getRemoteUsernameFromURL(url) {
  if (url.includes('https')) {
    return url.split('/').reverse()[1].trim();
  }

  return url.split(':')[1].split('/')[0].trim();
}

function getRemoteUsername() {
  return getRemoteUsernameFromURL(getRemoteURL());
}

function getRemoteRepositoryNameFromURL(url) {
  return url.split('/').pop().trim().replace('.git', '');
}

function getRemoteRepositoryName() {
  return getRemoteRepositoryNameFromURL(getRemoteURL());
}

function pullAndRebase() {
  return (0, _child_process.execSync)('git pull --rebase', { encoding: 'utf8' });
}

function checkoutBranch(branchName) {
  return (0, _child_process.execSync)(`git checkout ${branchName}`, { encoding: 'utf8' });
}

function createBranch(branchName) {
  return (0, _child_process.execSync)(`git checkout -b ${branchName}`, { encoding: 'utf8' });
}

function createCommit(message) {
  return (0, _child_process.execSync)(`git commit -m "${message}"`, { encoding: 'utf8' });
}
//# sourceMappingURL=git-cli.js.map