'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

let renderPRSummary = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* (pullrequest) {
    try {
      yield (0, _config.requireLogin)();

      const [statuses, activity] = yield _promise2.default.all([(0, _bitbucket.bitbucketRequest)(pullrequest.links.statuses.href).then(function ({ values }) {
        return values;
      }), (0, _bitbucket.bitbucketRequest)(pullrequest.links.activity.href).then(function ({ values }) {
        return values;
      })]);

      (0, _log.logPRHeader)(pullrequest);
      (0, _log.logPRApprovals)(parseUserApprovals(activity));

      if (statuses && statuses.length) {
        statuses.forEach(_log.logPRStatus);
      }

      if (pullrequest.description.trim().length) {
        (0, _log.logPRDescription)(pullrequest.description);
      }

      return true;
    } catch (e) {
      throw e;
    }
  });

  return function renderPRSummary(_x) {
    return _ref.apply(this, arguments);
  };
})();

let getPullRequestActions = (() => {
  var _ref2 = (0, _asyncToGenerator3.default)(function* (pr) {
    const activity = (yield (0, _bitbucket.bitbucketRequest)(pr.links.activity.href)).values;

    // based on activity, setup approve or unapprove link
    const hasApproved = parseUserApprovals(activity).includes(CURRENT_USERNAME);
    const approvalType = hasApproved ? 'unapprove' : 'approve';
    const approvalMethod = hasApproved ? 'delete' : 'post';

    return {
      checkout: function () {
        return _gitCli.pullAndRebase && (0, _gitCli.checkoutBranch)(pr.source.branch.name);
      },
      [approvalType]: function () {
        return (0, _bitbucket.bitbucketRequest)(pr.links.approve.href, {}, approvalMethod);
      },
      decline: function () {
        return (0, _bitbucket.bitbucketRequest)(pr.links.decline.href, {}, 'post');
      },
      // comment: () => promptComment(pr.id),
      // activity: () => bitbucketRequest(pr.links.activity.href),
      merge: function () {
        return (0, _bitbucket.bitbucketRequest)(pr.links.merge.href, {}, 'post');
      },
      quit: function () {}
    };
  });

  return function getPullRequestActions(_x2) {
    return _ref2.apply(this, arguments);
  };
})();

// not support with bitbucket API 2.0


let promptCreatePullRequest = (() => {
  var _ref3 = (0, _asyncToGenerator3.default)(function* () {
    const spinner = (0, _ora2.default)();
    try {
      yield (0, _config.requireLogin)();

      const currentBranch = (0, _gitCli.getBranch)();
      const prObj = {
        source: { branch: { name: currentBranch } },
        title: '',
        description: '',
        reviewers: []
      };
      const { createpr } = yield _inquirer2.default.prompt({
        type: 'confirm',
        name: 'createpr',
        message: `Would you like to create a pull request for your branch ${currentBranch}?`,
        validate: function (val) {
          return !!val;
        },
        filter: function (val) {
          return val.trim();
        },
        when: function () {
          return true;
        }
      });

      if (!createpr) {
        return true;
      }

      spinner.start('Preparing pull request...');
      const [repository, defaultReviewers = []] = yield _promise2.default.all([(0, _bitbucket.getRepository)(),
      // if default reviewers has an error, skip this step
      // by giving an empty list
      (0, _bitbucket.getRepositoryDefaultReviewers)().catch(function () {
        return [];
      })]);

      spinner.succeed('All set!');
      if (repository) {
        const {
          destination,
          closeBranch,
          title,
          description,
          reviewers = [],
          addReviewers
        } = yield _inquirer2.default.prompt([{
          type: 'input',
          name: 'destination',
          message: 'What branch should this pull request merge into?',
          default: repository.mainbranch.name,
          validate: function (val) {
            return !!val;
          },
          filter: function (val) {
            return val.trim();
          },
          when: function () {
            return true;
          }
        }, {
          type: 'confirm',
          name: 'closeBranch',
          message: 'Close branch on merge?',
          when: function () {
            return true;
          }
        }, {
          type: 'input',
          name: 'title',
          message: 'Pull request title:',
          default: currentBranch,
          validate: function (val) {
            return !!val;
          },
          filter: function (val) {
            return val.trim();
          },
          when: function () {
            return true;
          }
        }, {
          type: 'confirm',
          name: 'addDescription',
          message: 'Add pull request description?',
          when: function () {
            return true;
          }
        }, {
          type: 'editor',
          name: 'description',
          message: 'Description:',
          filter: function (val) {
            return val.trim();
          },
          when: function ({ addDescription }) {
            return addDescription;
          }
        }, {
          type: 'checkbox',
          name: 'reviewers',
          message: 'Select your reviewers:',
          choices: defaultReviewers.filter(function (u) {
            return u.username !== CURRENT_USERNAME;
          }).map(function (i) {
            return {
              name: i.display_name,
              value: i.username,
              checked: true
            };
          }),
          when: function () {
            return defaultReviewers && defaultReviewers.length;
          }
        }, {
          type: 'input',
          name: 'addReviewers',
          message: 'Add reviewers? (Enter usernames as csv)',
          filter: function (val) {
            return val.trim();
          }
        }]);

        const allReviewers = [...reviewers, ...addReviewers.split(',')].filter(function (i) {
          return i && i.trim().length;
        }).map(function (i) {
          return { username: i.trim() };
        });

        spinner.start('Creating your pull request...');
        const pullRequest = yield (0, _bitbucket.createPullRequest)((0, _extends3.default)({}, prObj, {
          destination: { branch: { name: destination } },
          title,
          description,
          close_source_branch: closeBranch,
          reviewers: allReviewers
        }));
        spinner.succeed('Pull request created!');

        (0, _log.logPRLink)(pullRequest.links.html.href);
      }

      return { success: true };
    } catch (e) {
      spinner.fail('Whoops!');
      const { fields } = e || {};
      const { source = [] } = fields || {};
      if (source && source.length) {
        source.forEach(function (m) {
          if (m.includes('branch not found')) {
            console.log(`\n${_chalk2.default.yellow('** Did you push your branch?')}\n`);
          }
          console.log(`${_chalk2.default.red('  Error: ' + m)}\n`);
        });
      }

      throw e;
    }
  });

  return function promptCreatePullRequest() {
    return _ref3.apply(this, arguments);
  };
})();

let runStatus = (() => {
  var _ref4 = (0, _asyncToGenerator3.default)(function* () {
    const spinner = (0, _ora2.default)();
    try {
      yield (0, _config.requireLogin)();

      spinner.start('Getting pull requests...');
      const pullrequests = yield (0, _bitbucket.getPullRequests)();
      if (pullrequests.length === 0) {
        spinner.succeed(`${_chalk2.default.green('All clear!')}`);
        console.log(_chalk2.default.cyan('No pull requests available.\n'));
        return true;
      }

      console.log('');
      spinner.succeed(`Found ${pullrequests.length} pull request(s)`);

      const prGroups = yield _promise2.default.all(pullrequests.map((() => {
        var _ref5 = (0, _asyncToGenerator3.default)(function* (pullrequest) {
          const [prstatus, activity] = yield _promise2.default.all([(0, _bitbucket.bitbucketRequest)(pullrequest.links.statuses.href).then(function ({ values }) {
            return values;
          }), (0, _bitbucket.bitbucketRequest)(pullrequest.links.activity.href).then(function ({ values }) {
            return values;
          })]);

          return {
            id: pullrequest.id,
            pullrequest,
            statuses: prstatus,
            approvals: parseUserApprovals(activity)
          };
        });

        return function (_x3) {
          return _ref5.apply(this, arguments);
        };
      })()));

      prGroups.sort(function (pr) {
        return pr.id;
      }).forEach(function ({ pullrequest, statuses, approvals }, index) {
        if (index === 0) {
          console.log(_chalk2.default.dim('===================================='));
        } else {
          console.log('-------------------------------------');
        }

        (0, _log.logPRHeader)(pullrequest);

        if (approvals.length) {
          console.log(`${_chalk2.default.green('\u2713')} Approved by ${approvals.join(', ')}\n`);
        } else {
          console.log(`${_chalk2.default.red('\u2717')} Not yet approved \n`);
        }
        if (statuses && statuses.length) {
          statuses.forEach(_log.logPRStatus);
        }

        (0, _log.logPRLink)(pullrequest.links.html.href);
      });

      return true;
    } catch (e) {
      spinner.fail('Whoops, something happened!');
      throw e;
    }
  });

  return function runStatus() {
    return _ref4.apply(this, arguments);
  };
})();

let promptPullRequestActions = (() => {
  var _ref6 = (0, _asyncToGenerator3.default)(function* (pullrequest) {
    const spinner = (0, _ora2.default)();
    try {
      spinner.start();
      const actions = yield getPullRequestActions(pullrequest);
      spinner.stop();
      const { action } = yield _inquirer2.default.prompt({
        type: 'list',
        name: 'action',
        message: 'What action would you like to perform?',
        choices: (0, _keys2.default)(actions).map(function (a) {
          return {
            name: a,
            value: actions[a]
          };
        }),
        validate: function (val) {
          return !!val;
        },
        when: function () {
          return true;
        }
      });

      if (action) {
        spinner.start();
        const data = yield action();
        spinner.succeed(`Done`);
        (0, _log.logPRLink)(pullrequest.links.html.href);
        if (data && data.values) {
          // data.values.map((i) => console.log('i - ', JSON.stringify(i)))
        }
      }

      return null;
    } catch (e) {
      spinner.fail('Whoops, something happened');
      throw e;
    }
  });

  return function promptPullRequestActions(_x4) {
    return _ref6.apply(this, arguments);
  };
})();

let promptPullRequestList = (() => {
  var _ref7 = (0, _asyncToGenerator3.default)(function* () {
    const spinner = (0, _ora2.default)();
    try {
      yield (0, _config.requireLogin)();

      spinner.start('Getting pull requests...');
      const list = yield (0, _bitbucket.getPullRequests)();

      if (list.length === 0) {
        spinner.succeed(`${_chalk2.default.green('All clear!')}`);
        console.log(_chalk2.default.cyan('No pull requests available.\n'));
        return true;
      }
      spinner.stop();

      const { pullrequest } = yield _inquirer2.default.prompt({
        type: 'list',
        name: 'pullrequest',
        message: 'Select a pull request?',
        choices: list.map(function (_ref8) {
          let {
            author, state, id, title } = _ref8,
              pr = (0, _objectWithoutProperties3.default)(_ref8, ['author', 'state', 'id', 'title']);
          return {
            name: `(${state}) #${id} by ${author.display_name} - ${title}`,
            value: (0, _extends3.default)({
              author,
              state,
              id,
              title
            }, pr)
          };
        }),
        validate: function (val) {
          return !!val;
        },
        when: function () {
          return true;
        }
      });

      spinner.start('Getting summary...');
      yield renderPRSummary(pullrequest);
      spinner.stop();

      return yield promptPullRequestActions(pullrequest);
    } catch (e) {
      throw e;
    }
  });

  return function promptPullRequestList() {
    return _ref7.apply(this, arguments);
  };
})();

exports.parseUserApprovals = parseUserApprovals;
exports.promptComment = promptComment;
exports.promptRepeatActionsList = promptRepeatActionsList;
exports.promptPullRequestCommand = promptPullRequestCommand;

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _config = require('./config');

var _bitbucket = require('./bitbucket');

var _gitCli = require('./git-cli');

var _log = require('./log');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CURRENT_USERNAME = (0, _config.getBitbucketUsername)();

function parseUserApprovals(activity) {
  return activity.filter(({ approval }) => approval).map(({ approval }) => approval.user.username);
}

function promptComment() {
  return _inquirer2.default.prompt({
    type: 'input',
    name: 'comment',
    message: 'Your comment:',
    validate: val => !!val,
    filter: val => val.trim(),
    when: () => true
  }).then(({ comment }) => (0, _bitbucket.addPullRequestComment)(comment)).catch(e => console.log(e));
}

function promptRepeatActionsList() {
  return _inquirer2.default.prompt({
    type: 'confirm',
    name: 'repeat',
    message: 'See actions again?',
    validate: val => !!val,
    when: () => true
  });
}

function promptPullRequestCommand({ create, status }) {
  if (!(0, _gitCli.getRemoteURL)().includes('bitbucket')) {
    console.log(_chalk2.default.cyan('hi pr currently supported with bitbucket only'));
    return _promise2.default.resolve(true);
  }

  if (create) {
    return promptCreatePullRequest();
  }

  if (status) {
    return runStatus();
  }

  return promptPullRequestList();
}
//# sourceMappingURL=pr.js.map